#ifndef _HEX_EDITOR_WIDGET_H_
#define _HEX_EDITOR_WIDGET_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include <qcombobox.h>
#include <qdialog.h> 

#include <kapp.h>
#include <kfm.h>

#include "dialog.h"
#include "hexbuffer.h"
#include "hexviewwidget.h"
#include "optiondialog.h"


class CHexEditorWidget : public QWidget
{
  Q_OBJECT

  public:
    CHexEditorWidget( QWidget *parent = 0, const char *name = 0 );
    ~CHexEditorWidget( void );
    void initialize( void );
    void writeConfiguration( KConfig &config );
    void readConfiguration( KConfig &config );

    void open( QString &url );
    bool selectDocument( QString &url );
    bool modified( void );

    inline int defaultTextWidth( void );
    inline CHexViewWidget *view( void );
    inline SDisplayLayout &layout( void );
    inline SDisplayLine &line( void );

  signals:
    void internetLoadingChanged( bool state );

  public slots:
    void fontChanged( void );
    void paletteChanged( void );
    void layoutChanged( const SDisplayLayout &layout );

    void setLineSize(const SDisplayLine &line );
    void setLayout( const SDisplayLayout &layout );
    void setCursor( const SDisplayCursor &cursor );
    void setColor( const SDisplayColor &color );
    void setFont( const SDisplayFont &font );
    void setMisc( const SDisplayMisc &misc );

    void newFile( void );
    void open( void );
    void insertFile( void );
    void stop( void );
    bool close( void );
    bool closeAll( void );
    void save( void );
    void saveAs( void );
    void reload( void );
    void print( void );
    void export( void );
    void undo( void );
    void redo( void );
    void benchmark( void );
    void copy( void );
    void copyPrimaryFormatted( void );
    void copyAllFormatted( void );
    void paste( void );
    void pasteNewFile( void );
    void cut( void );
    void selectAll( void );
    void unselect( void );
    void addBookmark( void );
    void removeBookmark( bool all );
    void replaceBookmark( void );
    void gotoBookmark( uint position );
    void gotoOffset( void );
    void find( void );
    void findAgain( void );
    void replace( void );
    void insertPattern( void );
    void options();

    void kfmFinished( void );
    void kfmError( int errCode, const char *errString );

  protected:
    void resizeEvent( QResizeEvent *e );

  private:
    enum action
    { 
      GET, 
      PUT 
    }; 

  private slots:  
    void exportData( SExportData &ed );

  private:
    bool querySave( void );
    void readURL( QString &url, bool insert );
    void writeURL( QString &url );
    bool readFile( const QString &diskPath, const QString &url, bool insert );
    bool writeFile( QString &diskPath );
    KFM *CreateKFMConnection( void );
    void DestroyKFMConnection( void );

    CHexBuffer *documentItem( QString &url );
    bool createBuffer( void );
    void removeBuffer( void );

  private:
    KFM     *mKfm;
    QString mKfmURL;
    QString mKfmTMP;
    action  mKfmAction;

    uint mUntitledCount;
    QList<CHexBuffer> mDocumentList;
    SDisplayState  mDisplayState;

    CHexViewWidget *mHexView;
    CGotoDialog    *mGotoDialog;
    CFindDialog    *mFindDialog;
    CReplaceDialog *mReplaceDialog;
    CInsertDialog  *mInsertDialog;
    CExportDialog  *mExportDialog;
    COptionDialog  *mOptionDialog;
};

inline int CHexEditorWidget::defaultTextWidth( void )
{
  return( mHexView->lineWidth() + mHexView->scrollBarWidth() );
}

inline CHexViewWidget *CHexEditorWidget::view( void )
{
  return( mHexView );
}

inline SDisplayLayout &CHexEditorWidget::layout( void )
{
  return( mDisplayState.layout );
}

inline SDisplayLine &CHexEditorWidget::line( void )
{
  return( mDisplayState.line );
}



#endif



