
// kpropdlg.cpp (C) 1998 by F. Zigterman

/***************************************************************

     Requires the Qt and KDE widget libraries, available at no cost at
     http://www.troll.no and http://www.kde.org respectively

     Copyright (C) 1997, 1998 Fester Zigterman ( fzr@dds.nl )

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************/


#include "kpropdlg.h"
#include "kapp.h"
#include "kpanner.h"

#include <qsplitter.h> 


#define QT_14

KPropDlg::KPropDlg( int dlgtype, int buttons, const char *title, 
		    QWidget *parent, const char *name, bool modal )
  : QDialog( parent, name, modal, WStyle_Customize|WStyle_DialogBorder )
{
  // Set window props
  setCaption( title );

  // Set some characteristics
  PageList = new QList<QWidget>;
  DlgType = dlgtype;
  Buttons = buttons;
  TreeWidth = 150;

  mActivePageWidget = 0;
  mActivePageIndex = -1;

  mTitleList.setAutoDelete( true );

  if( DlgType == TREE )
  {
    KPanner *panner = new KPanner( this );    
    panner->setSeparator( 25 );
    TreeList = new KTreeList( panner->child0() );
    
    (new QHBoxLayout( panner->child0() ) )->addWidget( TreeList );
    connect(TreeList, SIGNAL(highlighted(int)),this,SLOT(showPage(int)));

    rpane = panner->child1() ;
    VLayout = new QVBoxLayout( rpane );
    
    // Page title label
    Title = new QLabel( "Unnamed Dialog", rpane );
    Title->setFrameStyle( QFrame::Panel|QFrame::Raised );
    
    Title->setText( title );
    VLayout->addWidget( Title );

    PageFrame = new QFrame( rpane );
    PageFrame->setFrameStyle( QFrame::Panel|QFrame::Raised);
    PageFrame->installEventFilter( this );
    VLayout->addWidget( PageFrame );

    ButtonLayout = new QVBoxLayout( this );
    ButtonLayout->addWidget( panner, 0, AlignRight );
    QFrame *hline = new QFrame(this);
    hline->setFrameStyle(QFrame::Sunken|QFrame::HLine);
    hline->setFixedHeight(hline->sizeHint().height());
    ButtonLayout->addWidget( hline );
    
  }
  else if( DlgType == TABS )
  {
    TabControl = new KTabCtl( this );
    //ButtonWidget = new QLabel( this );
    ButtonLayout = new QVBoxLayout( this );
    ButtonLayout->addWidget( TabControl, 0, AlignRight );
    //ButtonLayout->addWidget( ButtonWidget, 0, AlignRight );
  }
  else
  {
    VLayout = new QVBoxLayout( this );
    
    // Page title label
    Title = new QLabel( "Unnamed Dialog", this );
    Title->setFrameStyle( QFrame::Panel|QFrame::Raised );
    Title->setFixedHeight( 20 );
    Title->setText( title );
    VLayout->addWidget( Title );

    PageFrame = new QFrame( this );
    PageFrame->setFrameStyle( QFrame::Panel|QFrame::Raised);
    PageFrame->installEventFilter( this );
    VLayout->addWidget( PageFrame );

    QFrame *hline = new QFrame(this);
    hline->setFrameStyle(QFrame::Sunken|QFrame::HLine);
    hline->setFixedHeight(hline->sizeHint().height());
    ButtonLayout->addWidget( hline );
  }

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 5, 5 );
  mHelpButton = buttonBox->addButton( i18n("&Help"), false );
  buttonBox->addStretch();
  mDefaultButton = buttonBox->addButton( i18n("&Default"), false );
  mApplyButton  = buttonBox->addButton( i18n("&Apply"), false );
  mCancelButton = buttonBox->addButton( i18n("&Close"), false );
  buttonBox->layout();
  mApplyButton->setDefault( true );
  buttonBox->setMinimumSize( mCancelButton->sizeHint() );
  buttonBox->setFixedHeight( buttonBox->sizeHint().height() );
  ButtonLayout->addWidget( buttonBox );

  Title->setFixedHeight( mCancelButton->sizeHint().height() );

  connectButtons();
}

void KPropDlg::connectButtons( int buttons )
{
  //
  // connect the button signals to their handlers
  //
  connect(mDefaultButton, SIGNAL(clicked()), SLOT(slotDefault()));
  connect(mApplyButton, SIGNAL(clicked()), SLOT(slotApply()));
  connect(mCancelButton, SIGNAL(clicked()), SLOT(slotCancel()));

}

KPropDlg::~KPropDlg()
{
}

QFrame *KPropDlg::addPage( int parentindex, 
			   const char *title, const char *longTitle,
			   int position )
{
  QVBoxLayout *templayout;
  QFrame *page; 


  if( DlgType == TABS )
    page = new QFrame( TabControl, title );
  else
    page = new QFrame( PageFrame, title );


  page->setFrameStyle( QFrame::NoFrame);
  page->setEnabled( FALSE );
  page->hide();
	
  if( DlgType == TABS )
  {
    TabControl->addTab( page, title );
  }
  else
  {
    if( DlgType == TREE )
    {
      TreeList->insertItem( title, 0, parentindex );
    }
    templayout = new QVBoxLayout( PageFrame,2 );
    templayout->addWidget( page );
    templayout->activate();
		
    page->lower();
  }
  PageList->append( page );

  QString *str = new QString( longTitle == 0 ? title : longTitle );
  mTitleList.append( str );

  return page;
}

void KPropDlg::showPage( QWidget *w )
{	
  if( w == 0 )
  {
    return;
  }

  //
  // Bring the new page up front and hode the old
  //
  w->raise();
  if( mActivePageWidget != 0 )
  {
    mActivePageWidget->setEnabled( false );
    mActivePageWidget->hide();
  } 

  mActivePageWidget = w;
  mActivePageWidget->setEnabled( true );
  mActivePageWidget->show();
	
  //
  // Update treelist and local header text
  //
  int index = PageList->findRef( w );
  if( DlgType == TREE )
  {
    QString *str = mTitleList.at(index);
    Title->setText( str == 0 ? w->name() : str->data() );
    TreeList->setCurrentItem(index);
  }
	
}

bool KPropDlg::showPage( int index )
{
  //
  // Locate page.
  //
  QWidget *page = PageList->at( index );
  if( page == 0 ) 
  {	
    warning("Error: No page with index %d", index );
    return( false );
  }
  else
  {
    mActivePageIndex = index;
    showPage( page );
    return( true );
  }
}


int KPropDlg::activePage( void )
{
  return( mActivePageIndex );
}






bool KPropDlg::removePage( char *page, int index )
{
  warning("Removing pages is not yet implemented.");
  return FALSE;
}

void KPropDlg::setButtonsEnabled( int types )
{
  ActiveButtons = types;
  /*
  PrevButton->setEnabled(  types&PREV  != 0 );
  NextButton->setEnabled(  types&NEXT  != 0 );
  ApplyButton->setEnabled( types&APPLY != 0 );
  OKButton->setEnabled(    types&OK    != 0 );
  CancelButton->setEnabled(types&CANCEL!= 0 );
  */
}

int  KPropDlg::buttonsEnabled()
{
  return ActiveButtons;
}

void KPropDlg::setButton( int type, const char *text )
{
}


void KPropDlg::slotDefault( void ) 
{
  emit defaultClicked();
}

void KPropDlg::slotApply( void ) 
{
  emit applyClicked();
}

void KPropDlg::slotCancel( void ) 
{
  reject();
}



int KPropDlg::pageListStatus()
{
  int status = 0;
  if( ! PageList ) return NOLIST;
  if( ! PageList->count() ) return NOPAGES;
  if( ! PageList->current() ) return NOCURRENT;
  if( PageList->current() == PageList->getLast() ) status = LASTPAGE;
  if( PageList->current() == PageList->getFirst() ) status = FIRSTPAGE;
  return status;
}

void KPropDlg::acceptConfig()
{
  // Write configuration and exit dialog cleanly.
  emit setConfig();
  kapp->getConfig()->sync();
  emit configChanged();
  accept();
}

void KPropDlg::cancelConfig()
{
  // Don't write configuration and cancel dialog.
  reject();
}

void KPropDlg::applyConfig()
{
  // Write configuration and return to dialog.
  //printf("applyConfig\n");
  emit setConfig();
  emit configChanged();
}

void KPropDlg::connectConfig( QObject * propconf )
{
  connect( this, 
	   SIGNAL( setConfig() ), 
	   propconf, 
	   SLOT( setConfig() ) 
	   );

  connect( this, 
	   SIGNAL( getConfig() ), 
	   propconf, 
	   SLOT( getConfig() ) 
	   );
}

bool KPropDlg::eventFilter( QObject *obj, QEvent *ev )
{
  if( ev->type() == Event_Resize )
  {
    adjustPageHeight();
    if( ( (QResizeEvent*)ev )->size().height() < minimumPageHeight )
      PageFrame->setMinimumHeight( minimumPageHeight );
  }
  return FALSE;
}

void KPropDlg::adjustPageHeight()
{
  // calculate the minimum size
  minimumPageHeight = 0;
  int index = PageList->at();
  for( PageList->first(); PageList->current(); PageList->next() )
  {
    minimumPageHeight = QMAX( minimumPageHeight, 
			      PageList->current()->minimumSize().height() );
      
  }
  PageFrame->setMinimumHeight( (minimumPageHeight += 10) );
  PageList->at(index);
}
