#include <qbttngrp.h>
#include <qfont.h>
#include <qlayout.h>
#include <qlabel.h>

#include <kapp.h>

#include "optiondialog.h"


COptionDialog::COptionDialog( const char *title, QWidget *parent, 
				char *name, bool modal )
  : KPropDlg( KPropDlg::TREE, KPropDlg::OK | 
	      KPropDlg::CANCEL | KPropDlg::APPLY,
	      title, 0, name, modal )
{
  const char *longTitle;

  longTitle = i18n("Layout and data appearance in primary editor view.");
  mFrame[ page_layout ]   = addPage( -1, i18n("Layout"), longTitle );
  longTitle = i18n("Cursor properties for editor area.");
  mFrame[ page_cursor ] = addPage( -1, i18n("Cursor"), longTitle );
  longTitle = i18n("Color selection. Use standard or make yor own mix.");
  mFrame[ page_color ]  = addPage( -1, i18n("Colors"), longTitle );
  longTitle = i18n("Font selection. Editor can only use a fixed font.");
  mFrame[ page_font ]   = addPage( -1, i18n("Font"), longTitle );
  longTitle = i18n("Various properties.");
  mFrame[ page_misc ] = addPage( -1, i18n("Miscellaneous"), longTitle );


  SetupLayoutPage();
  SetupCursorPage();
  SetupColorPage();
  SetupFontPage();
  SetupMiscPage();

  connect( this, SIGNAL(defaultClicked()), SLOT(slotDefault()));
  connect( this, SIGNAL(applyClicked()), SLOT(slotApply()));


  setMinimumSize( 500, 370 );
  adjustSize();
  showPage( page_layout );
}

COptionDialog::~COptionDialog( void )
{
}




void COptionDialog::SetupLayoutPage( void )
{
  const char *text;
  QVBoxLayout *topLayout = new QVBoxLayout( mFrame[ page_layout ], 5 );

  mLayout.lineSize = new CLineSizeWidget( mFrame[ page_layout ] );
  topLayout->addWidget( mLayout.lineSize );

  QFrame *page;
  QGridLayout *gbox;
  QLabel *label1, *label2;

  
  page = new QFrame( mFrame[ page_layout ] );
  if( page == 0 ) { return; }
  topLayout->addWidget( page );
  QVBoxLayout *vbox = new QVBoxLayout( page, 5 );
  text = i18n("&Lock line size (use scrollbar when required)"); 
  mLayout.lockLineCheck = new QCheckBox( text, page );
  mLayout.lockLineCheck->setFixedHeight( 
    mLayout.lockLineCheck->sizeHint().height() );
  vbox->addWidget( mLayout.lockLineCheck );
  text = i18n("Lock colu&mn at end of line"); 
  mLayout.lockColumnCheck = new QCheckBox( text, page );
  mLayout.lockColumnCheck->setFixedHeight( 
    mLayout.lockColumnCheck->sizeHint().height());
  vbox->addWidget( mLayout.lockColumnCheck );
  text = i18n("Show l&eft separator");
  mLayout.leftCheck = new QCheckBox( text, page );
  mLayout.leftCheck->setFixedHeight( mLayout.leftCheck->sizeHint().height() );
  vbox->addWidget( mLayout.leftCheck );
  text = i18n("Show &right separator");
  mLayout.rightCheck = new QCheckBox( text, page );
  mLayout.rightCheck->setFixedHeight( mLayout.rightCheck->sizeHint().height());
  vbox->addWidget( mLayout.rightCheck );
  vbox->activate();

  QButtonGroup *group = new QButtonGroup( mFrame[ page_layout ] );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );
  group->setTitle( "Margin sizes" );
  gbox = new QGridLayout( group, 3, 2, 5 );
  gbox->addRowSpacing( 0, 5 );
  mLayout.separatorSpin = new QSpinBox( group );
  mLayout.separatorSpin->setFixedHeight( 
    mLayout.separatorSpin->sizeHint().height() );
  mLayout.separatorSpin->setRange( 0, 20 );
  gbox->addWidget( mLayout.separatorSpin, 1, 1 );
  mLayout.edgeSpin = new QSpinBox( group );
  mLayout.edgeSpin->setFixedHeight( mLayout.edgeSpin->sizeHint().height() );
  mLayout.edgeSpin->setRange( 0, 20 );
  gbox->addWidget( mLayout.edgeSpin, 2, 1 );
  text = i18n("&Separator margin width [pixels]");
  label1 = new QLabel( mLayout.separatorSpin, text, group );
  gbox->addWidget( label1, 1, 0 );
  text = i18n("&Edge margin width [pixels]");
  label2 = new QLabel( mLayout.edgeSpin, text, group );
  gbox->addWidget( label2, 2, 0 );
  gbox->activate();


  topLayout->addStretch();
  topLayout->activate();
}


void COptionDialog::SetupCursorPage( void )
{
  const char *text;
  QVBoxLayout *topLayout = new QVBoxLayout( mFrame[ page_cursor ], 5 );

  QGroupBox *group = new QGroupBox( mFrame[ page_cursor ] );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );
  group->setTitle( i18n("Blinking") );
  group->setFrameStyle( QFrame::Box|QFrame::Sunken );
  QVBoxLayout *vbox = new QVBoxLayout( group, 5 );
  vbox->addSpacing( group->fontMetrics().height() );
  text = i18n("Do not b&link");
  mCursor.blinkCheck = new QCheckBox( text, group );
  mCursor.blinkCheck->setFixedHeight(mCursor.blinkCheck->sizeHint().height());
  connect( mCursor.blinkCheck, SIGNAL(toggled(bool)),
	   SLOT( slotBlinkIntervalCheck(bool)));
  vbox->addWidget( mCursor.blinkCheck );
  
  QHBoxLayout *hbox = new QHBoxLayout();
  if( hbox == 0 ) { return; }
  vbox->addLayout( hbox );
  mCursor.blinkSpin = new QSpinBox( group );
  mCursor.blinkSpin->setFixedHeight( mCursor.blinkSpin->sizeHint().height() );
  mCursor.blinkSpin->setRange( 100, 1000 );
  mCursor.blinkSpin->setSteps( 100, 100 );
  mCursor.blinkSpin->setValue( 500 );
  text = i18n("&Blink interval [ms]" );
  mCursor.blinkLabel = new QLabel( mCursor.blinkSpin, text, group );
  hbox->addWidget( mCursor.blinkLabel );
  hbox->addWidget( mCursor.blinkSpin );
  vbox->activate();

  topLayout->addStretch();

  group = new QGroupBox( mFrame[ page_cursor ] );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );
  group->setTitle( i18n("Shape") );
  group->setFrameStyle( QFrame::Box|QFrame::Sunken );
  vbox = new QVBoxLayout( group, 5 );
  vbox->addSpacing( group->fontMetrics().height() );
  text = i18n("Always &use block (rectangular) cursor");
  mCursor.blockCheck = new QCheckBox( text, group );
  mCursor.blockCheck->setFixedHeight( mCursor.blockCheck->sizeHint().height());
  connect( mCursor.blockCheck, SIGNAL(toggled(bool)),
	   SLOT( slotBlockCursorCheck(bool)));
  vbox->addWidget( mCursor.blockCheck );
  text = i18n("Use &thick cursor in insert mode");
  mCursor.thickCheck = new QCheckBox( text, group );
  mCursor.thickCheck->setFixedHeight( mCursor.thickCheck->sizeHint().height());
  vbox->addWidget( mCursor.thickCheck );
  vbox->activate();

  topLayout->addStretch();

  QButtonGroup *buttonGroup = new QButtonGroup( mFrame[ page_cursor ] );
  if( buttonGroup == 0 ) { return; }
  topLayout->addWidget( buttonGroup );
  buttonGroup->setTitle( i18n("Cursor behaviour when editor looses focus") );
  vbox = new QVBoxLayout( buttonGroup, 5 );
  vbox->addSpacing( buttonGroup->fontMetrics().height() );
  mCursor.stopRadio = new QRadioButton( "radio1", buttonGroup );
  text = i18n("&Stop blinking (if blinking is enabled)");
  mCursor.stopRadio->setText( text );
  mCursor.stopRadio->setMinimumSize( mCursor.stopRadio->sizeHint() );
  vbox->addWidget( mCursor.stopRadio );
  mCursor.hideRadio = new QRadioButton( "radio2", buttonGroup );
  mCursor.hideRadio->setText(i18n("H&ide") );
  mCursor.hideRadio->setMinimumSize( mCursor.hideRadio->sizeHint() );
  vbox->addWidget( mCursor.hideRadio );
  mCursor.nothingRadio = new QRadioButton( "radio3", buttonGroup );
  mCursor.nothingRadio->setText(i18n("Do &nothing") );
  mCursor.nothingRadio->setMinimumSize( mCursor.nothingRadio->sizeHint() );
  vbox->addWidget( mCursor.nothingRadio );
  vbox->activate();

  topLayout->addStretch();
  topLayout->activate();
}

void COptionDialog::SetupColorPage( void )
{
  const char *text;
  QVBoxLayout *topLayout = new QVBoxLayout( mFrame[ page_color ], 5 );

  QLabel *label1;
  QColor color[9] = { white, black, red, darkGreen, black, blue, black, 
		      gray, red };

  QFrame *page = new QFrame( mFrame[ page_color ] );
  if( page == 0 ) { return; }
  topLayout->addWidget( page );
  QGridLayout *gbox = new QGridLayout( page, 2, 2, 5 );
  mColor.textBg = new KColorButton( color[0], page );
  mColor.textBg->setFixedHeight( mColor.textBg->sizeHint().height() );
  label1 = new QLabel( mColor.textBg, i18n("&Text background:"), page );
  label1->setFixedHeight( label1->sizeHint().height() );
  gbox->addWidget( label1, 0, 0 );
  gbox->addWidget( mColor.textBg, 1, 0 );
  gbox->activate();

  text = i18n("&Use system colors (as chosen in Control Center)");
  mColor.checkSystem = new QCheckBox( text, mFrame[ page_color ] );
  mColor.checkSystem->setFixedHeight(
    mColor.checkSystem->sizeHint().height());
  connect( mColor.checkSystem, SIGNAL(toggled(bool)),
	   SLOT( slotColorSystem(bool)));
  topLayout->addWidget( mColor.checkSystem );

  page = new QFrame( mFrame[ page_color ] );
  if( page == 0 ) { return; }
  page->setFrameStyle( QFrame::Box|QFrame::Sunken );
  topLayout->addWidget( page );
  gbox = new QGridLayout( page, 8, 2, 5 );

  const char *labeltext[8] = 
  {
    i18n("&Even column foreground:"),
    i18n("O&ffset foreground:"),
    i18n("&Separator foreground:"),
    i18n("Cu&rsor foreground:"),
    i18n("&Odd column foreground:"),
    i18n("Secondary foreground:"),
    i18n("Inactive background:"),
    i18n("Cursor &background:"),
  };

  KColorButton *button[8];
  QLabel *label[8];

  for( uint i=0; i<8; i++ )
  {
    button[i] = new KColorButton( color[i+1], page );
    button[i]->setFixedHeight( button[i]->sizeHint().height() );
    label[i] = new QLabel( button[i], labeltext[i], page );
    label[i]->setFixedHeight( label[i]->sizeHint().height() );
    if( i < 4 )
    {
      gbox->addWidget( label[i], i*2, 0 );
      gbox->addWidget( button[i], (i*2) + 1, 0 );
    }
    else
    {
      gbox->addWidget( label[i], i*2 - 8, 1 );
      gbox->addWidget( button[i], (i*2) + 1 - 8, 1 );
    }
  }
  mColor.evenFg      = button[0];
  mColor.oddFg       = button[4];
  mColor.offsetFg    = button[1];
  mColor.secondaryFg = button[5];
  mColor.separatorFg = button[2];
  mColor.inactiveBg  = button[6];
  mColor.cursorFg    = button[3];
  mColor.cursorBg    = button[7];
  mColor.evenLabelFg      = label[0];
  mColor.oddLabelFg       = label[1];
  mColor.offsetLabelFg    = label[2];
  mColor.secondaryLabelFg = label[3];
  mColor.separatorLabelFg = label[4];
  mColor.inactiveLabelFg  = label[5];
  mColor.cursorLabelFg    = label[6];
  mColor.cursorLabelBg    = label[7];
  gbox->activate();

  topLayout->activate();
}




void COptionDialog::SetupFontPage( void )
{
  const char *text;
  QVBoxLayout *topLayout = new QVBoxLayout( mFrame[ page_font ], 5 );
  text = i18n("&Use system font (as chosen in Control Center)");
  mFont.checkSystem = new QCheckBox( text, mFrame[ page_font ] );
  mFont.checkSystem->setFixedHeight( mFont.checkSystem->sizeHint().height());
  connect( mFont.checkSystem, SIGNAL(toggled(bool)),
	   SLOT( slotFontSystem(bool)));
  topLayout->addWidget( mFont.checkSystem );

  QFrame *page = new QFrame( mFrame[ page_font ] );
  if( page == 0 ) { return; }
  page->setFrameStyle( QFrame::Box|QFrame::Sunken );
  topLayout->addWidget( page );

  QBoxLayout *hbox = new QHBoxLayout( page, 2 );
  mFont.chooser = new KFontChooser( page, "font" );
  hbox->addWidget( mFont.chooser );

  mFont.chooser->setFont( QFont( "courier", 14, QFont::Bold, FALSE ), true );

  topLayout->addStretch(10);
  topLayout->activate();
}


void COptionDialog::SetupMiscPage( void )
{
  const char *text;

  QVBoxLayout *topLayout = new QVBoxLayout( mFrame[ page_misc ], 5 );
 
  QFrame *page = new QFrame( mFrame[ page_misc ] );
  if( page == 0 ) { return; }
  topLayout->addWidget( page );
  QGridLayout *gbox = new QGridLayout( page, 2, 2, 5 );

  mMisc.undoSpin = new QSpinBox( page );
  mMisc.undoSpin->setFixedHeight( mMisc.undoSpin->sizeHint().height() );
  mMisc.undoSpin->setRange( 10, 10000 );
  mMisc.undoSpin->setSteps( 5, 5 );
  mMisc.undoSpin->setValue( 50 );

  QLabel *label = new QLabel( mMisc.undoSpin, i18n("&Undo history"), page );
  label->setFixedHeight( label->sizeHint().height() );
  gbox->addWidget( label, 0, 0 );
  gbox->addWidget( mMisc.undoSpin, 1, 0 );
  gbox->activate();

  text = i18n("Auto&matic copy to clipboard on complete selection");
  mMisc.autoCheck = new QCheckBox( text, mFrame[ page_misc ] );
  mMisc.autoCheck->setFixedHeight( mMisc.autoCheck->sizeHint().height() );
  topLayout->addWidget( mMisc.autoCheck );

  text = i18n("&Remember open files");
  mMisc.openCheck = new QCheckBox( text, mFrame[ page_misc ] );
  mMisc.openCheck->setFixedHeight( mMisc.openCheck->sizeHint().height() );
  topLayout->addWidget( mMisc.openCheck );

  QGroupBox *group = new QGroupBox( mFrame[ page_misc ] );
  if( group == 0 ) { return; }
  group->setTitle( i18n("Sounds") );
  topLayout->addWidget( group );
  QVBoxLayout *vbox = new QVBoxLayout( group, 5 );
  vbox->addSpacing( group->fontMetrics().height() );
  text = i18n("Make sound on &type (data input) error");
  mMisc.inputCheck = new QCheckBox( text, group );
  mMisc.inputCheck->setFixedHeight( mMisc.inputCheck->sizeHint().height() );
  vbox->addWidget( mMisc.inputCheck );
  text = i18n("Make sound on &fatal failure");
  mMisc.fatalCheck = new QCheckBox( text, group );
  mMisc.fatalCheck->setFixedHeight( mMisc.fatalCheck->sizeHint().height() );
  vbox->addWidget( mMisc.fatalCheck );
  vbox->activate();

  topLayout->addStretch(10);
  topLayout->activate();
}


void COptionDialog::slotBlinkIntervalCheck( bool state )
{
  state = state == true ? false : true;
  mCursor.blinkSpin->setEnabled( state );
  mCursor.blinkLabel->setEnabled( state );
}

void COptionDialog::slotBlockCursorCheck( bool state )
{
  state = state == true ? false : true;
  mCursor.thickCheck->setEnabled( state );
}

void COptionDialog::slotFontSystem( bool state )
{
  mFont.chooser->setEnabled( state == true ? false : true );
}

void COptionDialog::slotColorSystem( bool state )
{
  state = state == true ? false : true;
  mColor.evenFg->setEnabled( state );
  mColor.oddFg->setEnabled( state );
  mColor.offsetFg->setEnabled( state );
  mColor.secondaryFg->setEnabled( state );
  mColor.separatorFg->setEnabled( state );
  mColor.inactiveBg->setEnabled( state );
  mColor.cursorFg->setEnabled( state );
  mColor.cursorBg->setEnabled( state );
  mColor.evenLabelFg->setEnabled( state );
  mColor.oddLabelFg->setEnabled( state );
  mColor.offsetLabelFg->setEnabled( state );
  mColor.secondaryLabelFg->setEnabled( state );
  mColor.separatorLabelFg->setEnabled( state );
  mColor.inactiveLabelFg->setEnabled( state );
  mColor.cursorLabelFg->setEnabled( state );
  mColor.cursorLabelBg->setEnabled( state );
}

void COptionDialog::slotDefault( void )
{
  switch( activePage() )
  {
    case page_layout:
    {
      SDisplayLayout layout; 
      SDisplayLine line; 
      setLayout( layout, line );
    }
    break;

    case page_cursor:
    {
      SDisplayCursor cursor;
      setCursor( cursor );
    }
    break;

    case page_color:
    {
      SDisplayColor color;
      setColor( color );
    }
    break;

    case page_font:
    {
      SDisplayFont font;
      setFont( font );
    }
    break;

    case page_misc:
    {
      SDisplayMisc misc;
      setMisc( misc );
    }
    break;
  }
}


void COptionDialog::slotApply( void )
{
  switch( activePage() )
  {
    case page_layout:
    {
      mLayout.lineSize->getValue( mDisplayState.line );
      emit lineSizeChoise( mDisplayState.line );

      SDisplayLayout &l = mDisplayState.layout;
      l.lockLine = mLayout.lockLineCheck->isChecked();
      l.lockColumn = mLayout.lockColumnCheck->isChecked();
      l.leftSeparatorWidth = mLayout.leftCheck->isChecked() == true ? 1 : 0;
      l.rightSeparatorWidth = mLayout.rightCheck->isChecked()== true ? 1 : 0;
      l.separatorMarginWidth = mLayout.separatorSpin->value();
      l.edgeMarginWidth = mLayout.edgeSpin->value();
      emit layoutChoise( mDisplayState.layout );
    }
    break;


    case page_cursor:
    {
      SDisplayCursor &c = mDisplayState.cursor;      
      c.alwaysVisible    = mCursor.blinkCheck->isChecked();
      c.interval         = mCursor.blinkSpin->value();
      c.alwaysBlockShape = mCursor.blockCheck->isChecked();
      c.thickInsertShape = mCursor.thickCheck->isChecked();
      c.focusMode        = cursorFocusMode();
      emit cursorChoise( mDisplayState.cursor );
    }
    break;

    case page_color:
    {
      SDisplayColor &c = mDisplayState.color;
      c.useSystemColor = mColor.checkSystem->isChecked();
      c.textBg = mColor.textBg->color();
      c.inactiveBg = mColor.inactiveBg->color();
      c.primaryFg[0] = mColor.evenFg->color();
      c.primaryFg[1] = mColor.oddFg->color();
      c.offsetFg = mColor.offsetFg->color();
      c.secondaryFg = mColor.secondaryFg->color();
      c.leftSeparatorFg = mColor.separatorFg->color();
      c.rightSeparatorFg = mColor.separatorFg->color();
      c.cursorBg = mColor.cursorBg->color();
      c.cursorFg = mColor.cursorFg->color();
      emit colorChoise( mDisplayState.color );
    }
    break;

    case page_font:
    {
      SDisplayFont &f = mDisplayState.font;
      f.useSystemFont = mFont.checkSystem->isChecked();
      f.localFont = mFont.chooser->font();
      emit fontChoise( mDisplayState.font );
    }
    break;

    case page_misc:
    {
      SDisplayMisc &m = mDisplayState.misc;
      m.undoLevel = mMisc.undoSpin->value();
      m.autoCopyToClipboard = mMisc.autoCheck->isChecked();
      m.remeberFiles = mMisc.openCheck->isChecked();
      m.inputSound = mMisc.inputCheck->isChecked();
      m.fatalSound = mMisc.fatalCheck->isChecked();
      emit miscChoise( mDisplayState.misc );
    }
    break;
  }

}

void COptionDialog::setLayout( SDisplayLayout &layout, SDisplayLine &line )
{
  mDisplayState.line = line;
  mDisplayState.layout = layout;

  mLayout.lineSize->setValue( line );
  mLayout.lockLineCheck->setChecked( layout.lockLine );
  mLayout.lockColumnCheck->setChecked( layout.lockColumn );
  mLayout.leftCheck->setChecked( layout.leftSeparatorWidth == 0 ?false:true );
  mLayout.rightCheck->setChecked( layout.rightSeparatorWidth == 0 ?false:true);
  mLayout.separatorSpin->setValue( layout.separatorMarginWidth );
  mLayout.edgeSpin->setValue( layout.edgeMarginWidth );
}


void COptionDialog::setCursor( SDisplayCursor &cursor )
{
  mDisplayState.cursor = cursor;
 
  mCursor.blinkCheck->setChecked( cursor.alwaysVisible );
  mCursor.blinkSpin->setValue( cursor.interval );
  mCursor.blockCheck->setChecked( cursor.alwaysBlockShape );
  mCursor.thickCheck->setChecked( cursor.thickInsertShape );
  bool state = cursor.focusMode == SDisplayCursor::stopBlinking ? true : false;
  mCursor.stopRadio->setChecked( state );
  state = cursor.focusMode == SDisplayCursor::hide ? true : false;
  mCursor.hideRadio->setChecked( state );
  state = cursor.focusMode == SDisplayCursor::ignore ? true : false;
  mCursor.nothingRadio->setChecked( state );
}


void COptionDialog::setColor( SDisplayColor &color )
{
  mDisplayState.color = color;

  mColor.checkSystem->setChecked( color.useSystemColor );
  mColor.textBg->setColor(color.textBg);
  mColor.inactiveBg->setColor(color.inactiveBg);
  mColor.evenFg->setColor(color.primaryFg[0]);
  mColor.oddFg->setColor(color.primaryFg[1]);
  mColor.offsetFg->setColor(color.offsetFg);
  mColor.secondaryFg->setColor(color.secondaryFg);
  mColor.separatorFg->setColor(color.leftSeparatorFg);
  mColor.cursorBg->setColor(color.cursorBg);
  mColor.cursorFg->setColor(color.cursorFg);
}


void COptionDialog::setFont( SDisplayFont &font )
{
  mDisplayState.font = font;

  mFont.checkSystem->setChecked( font.useSystemFont );
  mFont.chooser->setFont( font.localFont, true );
}


void COptionDialog::setMisc( SDisplayMisc &misc )
{
  mDisplayState.misc = misc;

  mMisc.undoSpin->setValue( misc.undoLevel );
  mMisc.autoCheck->setChecked( misc.autoCopyToClipboard );
  mMisc.openCheck->setChecked( misc.remeberFiles );
  mMisc.inputCheck->setChecked( misc.inputSound );
  mMisc.fatalCheck->setChecked( misc.fatalSound );
}


void COptionDialog::setState( SDisplayState &state )
{
  setLayout( state.layout, state.line );
  setCursor( state.cursor );
  setColor( state.color );
  setFont( state.font );
  setMisc( state.misc );
}


SDisplayCursor::EFocusMode COptionDialog::cursorFocusMode( void )
{
  if( mCursor.stopRadio->isChecked() == true )
  {
    return( SDisplayCursor::stopBlinking );
  }
  else if( mCursor.hideRadio->isChecked() == true )
  {
    return( SDisplayCursor::hide );
  }
  else
  {
    return( SDisplayCursor::ignore );
  }
}


