#ifndef _TOPLEVEL_H_
#define _TOPLEVEL_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <kapp.h>
#include <kstdaccel.h>
#include <ktmainwindow.h>

#include "hexbuffer.h"
#include "hexmanagerwidget.h"

class KHexEdit : public KTMainWindow
{
  enum EMenuConstant
  {
    permission_readonly = 1,
    permission_allowResize,
    view_hexadecimal,
    view_octal,
    view_binary,
    view_text,
    view_offset_area,
    view_text_area,
    view_offset_as_decimal,
    view_case_data,
    view_case_offset,
    view_lock_width,
    option_toolbar,
    option_statusbar,
    option_utility,
    option_edit,
    toolbar_Copy,
    toolbar_Cut,
    toolbar_Undo,
    toolbar_Redo,
    toolbar_InternetStop,
  };

  Q_OBJECT

  public:
    KHexEdit( void );
    ~KHexEdit( void );

    inline bool isValid( void );
    void open( QStrList &fileList );
    void initialize( void );

  public slots:
    void dropAction( KDNDDropZone *zone );
    void recentFile( int id );
    void newWindow( void );
    void pasteNewWindow( void );
    void closeWindow( void );
    void closeProgram( void );
    void statusBarPressed( int id );

    void internetLoadingChanged( bool state );
    void cursorChanged( SCursorState &state );
    void fileModified( SFileState &state );
    void layoutChanged( const SDisplayLayout &layout );
    void inputModeChanged( const SHexInputMode &mode );
    void bookmarkChanged( const QList<SCursorOffset> &list );

    void fileActive( QString &url );
    void fileClosed( QString &url );

    void readConfiguration( void );
    void writeConfiguration( void );

    void editMode( CHexBuffer::EEditMode editMode );
    void textWidth( uint width );

    void viewMenuCB( int id );
    void bookmarkMenuCB( int id );
    void bookmarkPopupCB( int id );
    void documentMenuCB( int id );
    void optionMenuCB( int id );
   
  protected:
    virtual bool queryExit( void );
    virtual bool queryClose( void );
   
  private:
    void addRecentFile( QString &fileName );
    bool closeAllWindow( void );
    void setUndoState( uint undoState );
    void setSelectionState( bool selectionValid );
    void addDocument( QString &fileName );
    void removeDocument( QString &fileName );

    void writeConfiguration( KConfig &config );
    void readConfiguration( KConfig &config );

    inline CHexEditorWidget *editor( void );
    inline CHexToolWidget *util( void );
    inline CHexViewWidget *textView( void );
    inline int acceleratorNumKey( uint index );
 
  private:
    static QList<KHexEdit> mWindowList;
    QList<QString> mDocumentList;
    CHexManagerWidget *mManager;
    KDNDDropZone *mZone;

    QPopupMenu *mFilePermission;
    QPopupMenu *mFile;
    QPopupMenu *mRecent;
    QPopupMenu *mEdit;
    QPopupMenu *mEditSpecial;
    QPopupMenu *mView;
    QPopupMenu *mCharSet;
    QPopupMenu *mBookmark;
    QPopupMenu *mDocuments;
    QPopupMenu *mTools;
    QPopupMenu *mOptions;
    QPopupMenu *mHelp;
    QPopupMenu *mEditorPopup;
    QPopupMenu *mBookmarkPopup;

    int mPermissionReadOnlyMB, mPermissionResizeMB;
    int mEditUndoMB, mEditRedoMB, mEditCopyMB, mEditCutMB;
    int mEditSpecialCopyText1MB, mEditSpecialCopyText2MB;
    int mViewHexMB, mViewOctalMB, mViewBinaryMB, mViewTextOnlyMB;
    int mViewOffsetColumnMB, mViewTextAreaMB, mViewOffsetAsDecimalMB;
    int mViewBigCaseDataMB, mViewBigCaseOffsetMB, mViewLockWidthMB;
    int mOptionToolBarMB, mOptionStatusBarMB, mOptionUtilityMB;
    int mPopHexMB, mPopOctalMB, mPopBinaryMB, mPopTextOnlyMB;
    int mPopReadOnlyMB, mPopResizeMB;

    bool mValid;
    SHexInputMode mInputMode;
    bool mShowToolBar;
    bool mShowStatusBar;
    bool mShowUtility;
    bool mSelectionValid;
    uint mUndoState;
    uint mViewMode;

};


inline bool KHexEdit::isValid( void )
{
  return( mValid );
}

inline CHexEditorWidget *KHexEdit::editor( void )
{
  return( mManager->editor() );
}

inline CHexViewWidget *KHexEdit::textView( void )
{
  return( mManager->editor()->view() );
}

inline CHexToolWidget *KHexEdit::util( void )
{
  return( mManager->util() );
}


int KHexEdit::acceleratorNumKey( uint index )
{
  static int keys[9] = 
  {
    Key_1, Key_2, Key_3, Key_4, Key_5, Key_6, Key_7, Key_8, Key_9
  };
  return( index >= 9 ? -1 : keys[index] );
}



#endif


